<?php

# check if session is valid
$newTarget = '/app/berogui/index.php?m=RebootReset';
include('/usr/local/www/berogui/includes/session.php');
unset($newTarget);

/**
 * @file
 * Factory Reset, Datenbank löschen und neu erzeugen
 */
chdir('../..');

include('./includes/variables.php');
include('./misc/database.php');
include('/usr/fallback/beroConf.php');

require_once('./modules/PstnHardware/PstnHardware.php');

/* ######## Functions ####### */

function get_config_to_sip_configuration ($extern, $port_type)
{
	// extern PSTN
	if ($extern == 0)
	{
		return "sip_ppi_user_setting=\${new source},from_sip_src_setting=ppi_user"; 
	}

	// intern PSTN - BRI
	if (strcmp($port_type, 'BRI') == 0)
	{
		return "sip_from_user_setting=\${account_username}"; // TODO: if pbx == 3CX, add ,from_sip_src_setting=from_display ... how he gets the 3CX ?
	}

	// intern PSTN - FXS
	return '';
}

function get_config_to_dialplan_configuration ($extern, $type)
{
	$port_type = 'FXS';

	if (strcmp($type, 'isdn') == 0)
	{
		$port_type = 'BRI'; // PRI can not be intern
	}

	return get_config_to_sip_configuration ($extern, $port_type);
}

function load_wizard_config($filename = '/tmp/wizard-voip.conf')
{
	$wizard_cfg = array();

	// load the file
	$file_loaded = parse_ini_file($filename, true);

	// general configuration
	$wizard_cfg['General']['Scenario'] = $file_loaded['Scenario'];
	$wizard_cfg['General']['cloud']	   = $file_loaded['cloud'];
	$wizard_cfg['General']['prefix']   = $file_loaded['prefix'];

	unset($file_loaded['Scenario']);
	unset($file_loaded['net']);
	unset($file_loaded['cloud']);

	foreach (array_keys($file_loaded) as $key)  // NB: the 'Port 0' is located to Extern PSTN modules
	{
		if (preg_match('/extern-pstn-([A-Z]*)/', $key, $matches))
		{
			$wizard_cfg['Pstn'][$matches[1]][0] = $file_loaded[$key];
			$wizard_cfg['Pstn'][$matches[1]][0]['config'] = get_config_to_sip_configuration(0, $matches[1]);
		}
		else if (preg_match('/intern-([0-9])-pstn-([A-Z]*)/', $key, $matches))
		{
			$wizard_cfg['Pstn'][$matches[2]][$matches[1]] = $file_loaded[$key];
			$wizard_cfg['Pstn'][$matches[2]][$matches[1]]['config'] = get_config_to_sip_configuration($matches[1], $matches[2]);
		}
		else if (strcmp($key, 'Dialplan-Extern') == 0)
		{
			$wizard_cfg['Dialplan'][0] = $file_loaded[$key];
			$wizard_cfg['Dialplan'][0]['config'] = get_config_to_dialplan_configuration(0, $file_loaded[$key]);
		}
		else if (preg_match('/Dialplan-Intern-([0-9])/', $key, $matches))
		{
			$wizard_cfg['Dialplan'][$matches[1]] = $file_loaded[$key];
			$wizard_cfg['Dialplan'][$matches[1]]['config'] = get_config_to_dialplan_configuration($matches[1], $file_loaded[$key]['toDirection']);
		}
		else 
		{
			$wizard_cfg[$key] = $file_loaded[$key];
		}
	}

	return $wizard_cfg;
}

function set_sip_profile_gateway_scenario ($ba, $sip_cfg, $pstn_cfg)
{
	$name 	   = sqlite_escape_string($sip_cfg['name']);
	$server    = sqlite_escape_string($sip_cfg['server']);
	$user      = sqlite_escape_string(isset($sip_cfg['user'])     ? $sip_cfg['user']   : '');
	$secret    = sqlite_escape_string(isset($sip_cfg['secret'])   ? $sip_cfg['secret'] : '');
	$register  = sqlite_escape_string(isset($sip_cfg['register']) ? 1 		   : 0 );
	$transport = sqlite_escape_string(strtolower($sip_cfg['transport']));
	$config    = sqlite_escape_string(get_config_to_sip_configuration(0, key($pstn_cfg)));

	$sql_str = "(name, sip_transport, sip_registrar, sip_domain, sip_proxy, user, secret, register, config) VALUES ('$name', '$transport', '$server', '$server', '$server', '$user', '$secret', '$register', '$config')";

	$ba -> insert_("INSERT INTO sip $sql_str");	
}

function set_sip_profile_gateway_pbx_scenario ($ba, $pbx_settings, $pstn_settings)
{
	$pbx_server   = $pbx_settings['server'];
	$pbx_port     = (isset($pbx_settings['port']) ? ":". $pbx_settings['port'] : '');

	$server = sqlite_escape_string($pbx_server . $pbx_port);

	$matchtype = 'touser';
	foreach ($pstn_settings as $port_type => $array_ports)
	{
		foreach (array_keys($array_ports) as $port)
		{
			$pstn_cfg = $pstn_settings[$port_type][$port];

			$name     = sqlite_escape_string("SIP-". $pstn_cfg['group']);
			$user     = sqlite_escape_string(isset($pstn_cfg['user'])     ? $pstn_cfg['user']   : '');
			$secret   = sqlite_escape_string(isset($pstn_cfg['secret'])   ? $pstn_cfg['secret'] : '');
			$register = sqlite_escape_string(isset($pstn_cfg['register']) ? 1		    : 0 );
			$config   = sqlite_escape_string(isset($pstn_cfg['config'])   ? $pstn_cfg['config'] : '');

			$sql_str = '';			

			// Extern
			if ($port == 0)
			{
				$sql_str = "(name, sip_registrar, sip_proxy, sip_domain, user, secret, register, config) VALUES ('$name', '$server', '$server', '$server', '$user', '$secret', '$register', '$config')";	

			}
			// Intern
			else
			{
				if (strcmp($port_type, 'BRI') == 0)
				{
					$sql_str = "(name, sip_registrar, sip_proxy, sip_domain, user, secret, register, matchtype, config) VALUES ('$name', '$server', '$server', '$server', '$user', '$secret', '$register', '$matchtype', '$config')";	
				}
				else
				{
					$sql_str = "(name, sip_registrar, sip_proxy, sip_domain, user, secret, register, matchtype) VALUES ('$name', '$server', '$server', '$server', '$user', '$secret', '$register', '$matchtype')";	
				}
			}

			$ba -> insert_("INSERT INTO sip $sql_str");
		}
	}
}

function set_pstn_hardware ($ba, $pstn_settings)
{
	foreach ($pstn_settings as $port_type => $parameters)
	{
		$keep_parameters = $parameters;	
		
		// port == 0 => Intern
		if (isset($keep_parameters[0]))
		{
			foreach (explode('-', $keep_parameters[0]['ports']) as $port)
			{
				$keep_parameters[$port] = $keep_parameters[0];
			}

			unset($keep_parameters[0]);
		}

		foreach (array_keys($keep_parameters) as $port)
		{
			$ptp	  = '';
			$tones 	  = '';
			$sql_str  = '';
			$pstn_cfg = $keep_parameters[$port];

			$group = sqlite_escape_string($pstn_cfg['group']);

			switch ($port_type)
			{
				case 'BRI':
				case 'PRI':
					$type = 'isdn';
					
					$ptp 	 = sqlite_escape_string(strcmp('PTP', $pstn_cfg['ptp']) == 0 ? 1 : 0);
					$ntte    = sqlite_escape_string(strtolower($pstn_cfg['ntte']));

					$ba -> update("UPDATE $type SET ntteHW='$ntte', ntteSW='$ntte', ptp='$ptp'  WHERE port='". sqlite_escape_string($port) ."'");	

					$tones   = sqlite_escape_string("[". $pstn_cfg['tones'] ."]");
					$country = sqlite_escape_string(isset($pstn_cfg['country']) ? $pstn_cfg['country'] : '');
					$city    = sqlite_escape_string(isset($pstn_cfg['city'])    ? $pstn_cfg['city']	   : '');
					$local   = sqlite_escape_string(isset($pstn_cfg['local'])   ? $pstn_cfg['local']   : '');
					//$oton    = sqlite_escape_string($pstn_cfg['oton']);  // we use the default value: oton=4

					$sql_str = ", tones='$tones', country_code='$country', city_code='$city', local_area_code='$local', config='oton=4'";
					break;
				case 'FXO':
				case 'FXS':
					$type  = 'analog';
					$tones = ", tones='". sqlite_escape_string("[". $pstn_cfg['tones'] ."]") . "'";

					$sql_str = $tones;
					break;
				case 'GSM':
				case 'LTE':
					$type = strtolower($port_type);
					break;
				default:
					// TODO: stop the factory reset if error
					return;
			}

			if (isset($pstn_cfg['t1e1']))
			{
				$ba -> update("UPDATE cas SET protocol='". sqlite_escape_string(strtolower($pstn_cfg['t1e1'])) ."', termination='$ntte' WHERE port_type='hdlc'");
			}

			$ba -> update("UPDATE $type SET gr='$group'$sql_str WHERE port='". sqlite_escape_string($port) ."'");
		}
	}
}

function set_dialplan_rules($ba, $dialplan_rules)
{
	$posId = 0;
	$activ = sqlite_escape_string('1');

	foreach ($dialplan_rules as $port => $parameters)
	{
		// init input variable
		$fromDirection  = sqlite_escape_string($parameters['fromDirection']);
		$toDirection	= sqlite_escape_string($parameters['toDirection']);
		$dest		= sqlite_escape_string('('. $parameters['dest'] .')');
		$newDest	= sqlite_escape_string("\\". $parameters['newDest']);
		$src		= sqlite_escape_string('('. $parameters['src'] .')');
		$newSrc		= sqlite_escape_string("\\". $parameters['newSrc']);
		$config		= sqlite_escape_string(isset($parameters['config']) ? $parameters['config'] : '');

		// SIP => PSTN
		$sip_ref = ($port == 0 ? 'd:' : 'a:');	// port == 0 => extern .... port != 0 => intern

		$fromId  = $sip_ref . $parameters['fromId'];
		$toId	 = "g:". $parameters['toId'];

		$posIdsql = sqlite_escape_string($posId);
		$sql_str  = "(fromDirection, toDirection, fromId, toId, dest, newDest, src, newSrc, posId, activ) VALUES ('$fromDirection', '$toDirection', '$fromId', '$toId', '$dest', '$newDest', '$src', '$newSrc', '$posIdsql', '$activ')";

		$ba -> insert_("INSERT INTO dialplan $sql_str");
		
		$posId++;

		// PSTN => SIP
		$fromId  = "g:". $parameters['toId'];
		$toId	 = "p:". $parameters['fromId'];

		$posIdsql = sqlite_escape_string($posId);
		if (strlen($config) == 0)
		{
			$sql_str = "(fromDirection, toDirection, fromId, toId, dest, newDest, src, newSrc, posId, activ) VALUES ('$toDirection', '$fromDirection', '$fromId', '$toId', '$dest', '$newDest', '$src', '$newSrc', '$posIdsql', '$activ')";
		}
		else
		{
	               $sql_str="(fromDirection, toDirection, fromId, toId, dest, newDest, src, newSrc, posId, activ, config) VALUES ('$toDirection', '$fromDirection', '$fromId', '$toId', '$dest', '$newDest', '$src', '$newSrc', '$posIdsql', '$activ', '$config')";
		}

		$ba -> insert_("INSERT INTO dialplan $sql_str");

		$posId++;	
	}
}

function update_lan_or_wan_network ($bc, $network_cfg, $which_settings, $root)
{
	// dhcp
	$dhcp = (isset($network_cfg[$which_settings['0']['name']]) ? $network_cfg[$which_settings['0']['name']] : 0); 
	$bc -> set($root, $which_settings['0']['id'], sqlite_escape_string($dhcp));

	if ($dhcp == 0)
	{
		// static
		for ($i = 1; $i < count($which_settings); $i++)
		{
			$i_str = (string)$i;
			if (isset($network_cfg[$which_settings[$i_str]['name']]))
			{
				$bc -> set($root, $which_settings[$i_str]['id'], sqlite_escape_string($network_cfg[$which_settings[$i_str]['name']]));
			}
		}
	}
}

function update_network_configuration ($bc, $network_cfg, $root = 'root')
{
	// delete old network settings
	$old_settings = array('lan-ipaddr', 'lan-netmask', 'lan-gateway','lan-nameserver', 'lan-mtu', 'lan-dhcp', 'wan-dhcp','wan-ipaddr', 'wan-netmask', 'wan-gateway', 'bonding-mode','bonding-miimon', 'bonding-updelay', 'bonding-downdelay');

	foreach ($old_settings as $old_setting)
	{
		$bc->delete($root, $field);
	}

	// get id new settings
	require_once('./modules/Wizard/network_wizard_popup.php');
	$network_wizard = new NetworkWizard();

	$new_settings = $network_wizard -> get_network_settings();

	// set lan or lan-wan mode
	$bc -> set($root, $new_settings['mode']['id'], sqlite_escape_string($network_cfg[$new_settings['mode']['name']]));

	// set lan network 
	update_lan_or_wan_network($bc, $network_cfg, $new_settings['lan'], $root);

	// set wan network
	if (strcmp($network_cfg[$new_settings['mode']['name']], 'lan-wan') == 0)
	{
		// remove lan gateway (correspond to the key '3' in the lan array)
		$bc->delete($root, $new_settings['lan']['3']['id']);

		update_lan_or_wan_network($bc, $network_cfg, $new_settings['wan'], $root);
	}

	// set dns configuration
	$nameserver = '';
	if (isset($network_cfg[$new_settings['dns']['0']['name']]))
	{
		$nameserver = $network_cfg[$new_settings['dns']['0']['name']] . (isset($network_cfg[$new_settings['dns']['1']['name']]) ? ";" . $network_cfg[$new_settings['dns']['1']['name']] : '');
		$bc -> set($root, $new_settings['dns']['0']['id'], sqlite_escape_string($nameserver));
	}

	if (isset($network_cfg[$new_settings['dns']['2']['name']]))
	{
		$bc -> set($root, $new_settings['dns']['2']['id'], sqlite_escape_string($network_cfg[$new_settings['dns']['2']['name']]));
	}
}

###############################

// init factory reset mode
$wizard_factory_reset = false;
$normal_factory_reset = true ;

$keep_cloud  = isset($_POST['keep_cloud_set']);
$keep_prefix = isset($_POST['keep_prefix_set']);

// init wizard factory reset
if (isset($_GET['wizard']) && $_GET['wizard'])
{
	$normal_factory_reset = false;
	$wizard_factory_reset = true ; 

	$wizard_cfg = load_wizard_config();

	$keep_cloud  = (isset($wizard_cfg['save_cloud'])  ? $wizard_cfg['save_cloud']  : false);
	$keep_prefix = (isset($wizard_cfg['save_prefix']) ? $wizard_cfg['save_prefix'] : false);
}

// init normal factory reset
if ($normal_factory_reset)
{
	$inserts=array();
	// keep sip-settings
	if (isset($_POST['keep_sip_set'])){
		$ba = new beroAri();
		$q = $ba->select('SELECT * FROM sip;');

		while ($entry=sqlite_fetch_array($q, SQLITE_ASSOC)){
			unset($fields);
			unset($values);
			foreach ($entry as $key => $val){
				$fields .= ((strlen($fields) == 0) ? '' : ',') . $key;
				$values .= ((strlen($values) == 0) ? '' : ',') . '\'' . sqlite_escape_string($val) . '\'';
			}
			$inserts[] = 'INSERT INTO sip (' . $fields . ') VALUES (' . $values . ');';
		}
	}

	//keep ACL Settings 
	if (isset($_POST['keep_set'])) {
		$ba = new beroAri();
		$q = $ba->select('SELECT * FROM acl;');

		while ($entry=sqlite_fetch_array($q, SQLITE_ASSOC)){
			unset($fields);
			unset($values);
			foreach ($entry as $key => $val){
				$fields .= ((strlen($fields) == 0) ? '' : ',') . $key;
				$values .= ((strlen($values) == 0) ? '' : ',') . '\'' . sqlite_escape_string($val) . '\'';
			}
			$inserts[] = 'INSERT INTO acl (' . $fields . ') VALUES (' . $values . ');';
		}
	}
}

// keep cloud-settings
if ($keep_cloud)
{
	$ba = new beroAri();
	$nameArray = array('proxyEnabled', 'proxyAddress', 'proxyPort', 'proxyType', 'proxyUser', 'proxySecret', 'cloudAddress');
	foreach ($nameArray as $name) 
	{
		$cloudSettings[$name] = $ba -> get($name);
	}
}

// keep prefix-settings
if ($keep_prefix)
{
	$ba = new beroAri();
	$nameArray = array('national_code', 'international_code');
	foreach ($nameArray as $name) 
	{
		$prefixSettings[$name] = $ba -> get($name);
	}
}

// delete database ari.db
exec('/bin/rm /usr/conf/ari.db');

//Create new db
$ba = new beroAri();

if ($ba -> is_error())
{
	echo '<script>alert("' . $ba -> error() .'");</script>';
}

// reset rootdb
$bc = new beroConf('root');
$bc->exec_sql("DELETE FROM root WHERE key NOT IN ('root-version', 'lan-mtu', 'lan-iface', 'lan-ipaddr', 'lan-netmask', 'lan-gateway', 'lan-dhcp', 'lan-mode', "
	. "'lan-ports', 'msp-iface', 'msp-ipaddr', 'app-image', 'app-status', 'ntphost', 'cloud_enable', 'cloud_key', 'lan-nameserver', 'vlan-enable', 'vlan-id', "
	. "'vlan-prio', 'wan-dhcp', 'wan-ipaddr', 'wan-netmask', 'wan-gateway', 'bonding-mode', 'bonding-miimon', 'bonding-updelay', 'bonding-downdelay');");

if (!($keep_cloud))
{
	$bc -> exec_sql("DELETE FROM root WHERE key IN ('cloud_enable', 'cloud_key');");
}

// wizard factory reset
if ($wizard_factory_reset)
{
	$wizard_cfg = load_wizard_config();

	// SIP profile(s)
	if ($wizard_cfg['General']['Scenario'] == 2)
	{
		set_sip_profile_gateway_pbx_scenario ($ba, $wizard_cfg['Pbx'], $wizard_cfg['Pstn']);
	}
	else
	{
		set_sip_profile_gateway_scenario ($ba, $wizard_cfg['Sip'], $wizard_cfg['Pstn']);
	}

	// PSTN configuration
	set_pstn_hardware ($ba, $wizard_cfg['Pstn']);

	// Dialplan Rules
	set_dialplan_rules($ba, $wizard_cfg['Dialplan']);

	// Update network settings
	if (isset($wizard_cfg['Network']))
	{
		update_network_configuration($bc, $wizard_cfg['Network']);
	}

	// set a default ntphost (if none is set, but configuration is static)
	if ($bc -> get('root', 'lan-dhcp') != 'yes') 
	{
		if (strlen($bc -> get('root', 'ntphost')) == 0) 
		{
			$bc -> set('root', 'ntphost', 'ntp0.fau.de');
		}
	}

	$bc -> set('root', 'msp-iface', 'eth1');
	$bc -> set('root', 'msp-ipaddr', 'auto');
	$bc -> set('root', 'app-status', 'running');
	$bc -> set('root', 'userapp-enabled', '1');
}

// normal factroy reset
else if ($normal_factory_reset)
{
	//keep network settings
	if (!isset($_POST['keep_set'])) {

		$delArray = array('lan-ipaddr', 'lan-netmask', 'lan-gateway', 'lan-nameserver', 'ntphost', 'vlan-enable', 'vlan-id', 
			'vlan-prio', 'lan-mode', 'wan-dhcp', 'wan-ipaddr', 'wan-netmask', 'wan-gateway', 'bonding-mode', 'bonding-miimon', 
			'bonding-updelay', 'bonding-downdelay');
		foreach ($delArray as $delItem) {
			$bc->delete('root', $delItem);
		}
		unset($delArray);

		$bc->set('root', 'lan-iface', 'eth0');
		$bc->set('root', 'lan-dhcp', 'yes');
		$bc->set('root', 'lan-mtu', '1500');
		$bc->set('root', 'beroAvahi-enabled', '1');
	}

	// set a default ntphost (if none is set, but configuration is static)
	if ($bc->get('root', 'lan-dhcp') != 'yes') {
		if (strlen($bc->get('root', 'ntphost')) == 0) {
			$bc->set('root', 'ntphost', 'ntp0.fau.de');
		}
	}

	//restore apps
	if (!isset($_POST['keep_app_set'])){
	
		exec('/bin/mount -o remount,rw /home/admin/');

		if ($handle = opendir('/home/admin/apps/')) {
			while (false !== ($file = readdir($handle))) {
				if (($file == '.') || ($file == '..')) {
					continue;
				}
				exec('/bin/rm -rf /home/admin/apps/' . $file);
				exec('/bin/rm -rf /home/admin/www/' . $file);
			}
			closedir($handle);
		}

		exec('/bin/rm -rf /usr/conf/userapp');
		exec('sync');
		exec('/bin/mount -o remount,ro /home/admin/');
	}

	$bc->set('root', 'msp-iface', 'eth1');
	$bc->set('root', 'msp-ipaddr', 'auto');
	$bc->set('root', 'app-status', 'running');
	$bc->set('root', 'userapp-enabled', '1');

	// restore settings
	foreach ($inserts as $sql) {
		$ba->insert_($sql);
	}

	// if E1/T1 module is plugged, we activate the module depending the choice made before to make the factory-reset
	if (isset($_POST['keep_hw_set'])) 
	{
	        if (!($_POST['id_upd']))
	        {
	                // we come directly from RebootReset.php page (modal has not been be shown)
	                // setting default value
	                $default_array = array( 'id_upd' => array ('0', '1'), 'type' => array('cashw', 'cas'), 
	                                        'interface_type1' => '0', 'interface_type2' => '0', 'interface_type3' => '0', 'interface_type4' => '0',
	                                        't1e11' => 'e1', 't1e12' => 'e1', 't1e13' => 'e1', 't1e14' => 'e1');
	
	                $_POST = array_merge($_POST, $default_array);
	        }
	
		$pstn_hw = new PstnHardware();
		$pstn_hw -> change_hardware_configuration($ba, $_POST);	
	
		// really useful ??
	        $ba -> set('activate', '3');
	}
}

// restore cloud-settings
if ($keep_cloud) 
{
	foreach ($cloudSettings as $key => $value) 
	{
		$ba -> set($key, sqlite_escape_string($value));
	}
}

// restore prefix-settings
if ($keep_prefix) 
{
	foreach ($prefixSettings as $key => $value) 
	{
		$ba -> set($key, sqlite_escape_string($value));
	}
}

// doesn't this belong into 'create_files.php'??
// reset api-password
@unlink('/usr/conf/apiusers.conf');
require_once('/usr/local/php/include/apiusers.Class.php');
$apiusers=new apiusers('admin');
$apiusers->createFile();

// reset password for user admin and deactivate account
exec('/usr/local/sbin/pwchange.sh admin admin');
exec('/bin/mount -oremount,rw /; /usr/bin/passwd -l admin; sync; /bin/mount -oremount,ro /');

echo '<script>window.location.href="../../includes/create_files.php"</script>';

?>
